@@ -132,8 +132,8 @@
 ##' head(out$des.rv)
 ##' 
 ##' bints <- binomial.twostage(margbin,data=data,
-##'      clusters=data$cluster,detail=0,var.par=0,
-##'      theta=c(2,1)/9,var.link=0,
+##'      clusters=data$cluster,detail=0,var.par=1,
+##'      theta=c(2,1),var.link=0,
 ##'      random.design=out$des.rv,theta.des=out$pardes)
 ##' summary(bints)
 ##' 
@@ -144,11 +144,10 @@
 ##' margbin <- glm(ybin~x,data=data,family=binomial())
 ##' 
 ##' bintwin <- binomial.twostage(margbin,data=data,
-##'      clusters=data$cluster,detail=1,var.par=0,
-##'      theta=c(2,1),
-##'      random.design=out$des.rv,theta.des=out$pardes)
+##'      clusters=data$cluster,detail=1,var.par=1,
+##'      theta=c(2,1),random.design=out$des.rv,theta.des=out$pardes)
 ##' summary(bintwin)
-##' concordance.twin.ace(bintwin)
+##' concordanceTwinACE(bintwin)
 ##' }
 ##' 
 ##' @keywords binomial regression 
@@ -186,7 +185,7 @@
 ##' @param pair.ascertained if pairs are sampled only when there are events in the pair i.e. Y1+Y2>=1. 
 ##' @param case.control if data is case control data for pair call, and here 2nd column of pairs are probands (cases or controls)
 ##' @param twostage default twostage=1, to fit MLE use twostage=0
-##' @param beta is starting value for beta for MLE version 
+##' @param beta is starting value for beta for MLE version
 binomial.twostage <- function(margbin,data=sys.parent(),
      score.method="fisher.scoring",Nit=60,detail=0,clusters=NULL,silent=1,weights=NULL,
      control=list(),theta=NULL,theta.des=NULL,var.link=0,var.par=1,var.func=NULL,
@@ -268,115 +267,131 @@ binomial.twostage <- function(margbin,data=sys.parent(),
     }                                                         
 # }}}
 
-    ratesim<-rate.sim; 
     if (is.null(beta)==TRUE & twostage==0) beta <- coef(margbin) else beta <- 0
+    ## }}}
+  dimbeta <- length(beta); 
 
-   if (!is.null(random.design)) { ### different parameters for Additive random effects # {{{
-     dep.model <- 3
-###     if (is.null(random.design)) random.design <- matrix(1,antpers,1); 
-     dim.rv <- ncol(random.design); 
-     if (is.null(theta.des)) theta.des<-diag(dim.rv);
-###     ptheta <- dimpar <- ncol(theta.des); 
- 
-###   if (dim(theta.des)[2]!=ncol(random.design)) 
-###   stop("nrow(theta.des)!= ncol(random.design),\nspecifies restrictions on paramters, if theta.des not given =diag (free)\n"); 
- } else { random.design <- matrix(0,1,1);  dim.rv <- 1; }
+###
+###   if (!is.null(random.design)) { ### different parameters for Additive random effects # {{{
+###     dep.model <- 3
+######     if (is.null(random.design)) random.design <- matrix(1,antpers,1); 
+###     dim.rv <- ncol(random.design); 
+###     if (is.null(theta.des)) theta.des<-diag(dim.rv);
+######     ptheta <- dimpar <- ncol(theta.des); 
+### 
+######   if (dim(theta.des)[2]!=ncol(random.design)) 
+######   stop("nrow(theta.des)!= ncol(random.design),\nspecifies restrictions on paramters, if theta.des not given =diag (free)\n"); 
+### } else { random.design <- matrix(0,1,1);  dim.rv <- 1; }
+###
+###    if (is.null(theta.des)==TRUE) ptheta<-1; 
+###    if (is.null(theta.des)==TRUE) theta.des<-matrix(1,antpers,ptheta) ###    else theta.des<-as.matrix(theta.des); 
+######    ptheta<-ncol(theta.des); 
+######    if (nrow(theta.des)!=antpers) stop("Theta design does not have correct dim");
+###
+###  if (!is.null(pairs)) { pair.structure <- 1; } else  pair.structure <- 0;  
+###  if (length(dim(theta.des))==3) ptheta<-dim(theta.des)[2] else if (length(dim(theta.des))==2) ptheta<-ncol(theta.des)
+###  if  (nrow(theta.des)!=antpers & dep.model!=3 & pair.structure==0 ) stop("Theta design does not have correct dim");
+###  if  (nrow(theta.des)!=nn      & dep.model!=3 & pair.structure==1 ) stop("Theta design does not have correct dim");
+###
+###   if (length(dim(theta.des))!=3) theta.des <- as.matrix(theta.des)
+######   theta.des <- as.matrix(theta.des)
+###
+###    dimbeta <- length(beta); 
+###    if (is.null(theta)==TRUE) {
+###        if (var.link==1) theta<- rep(-0.7,ptheta);  
+###        if (var.link==0) theta<- rep(exp(-0.7),ptheta);   
+###    }       
+###    if (length(theta)!=ptheta) theta<-rep(theta[1],ptheta); 
+###    theta.score<-rep(0,ptheta);Stheta<-var.theta<-matrix(0,ptheta,ptheta); 
+###
+###    if (maxclust==1) stop("No clusters, maxclust size=1\n"); 
+###
+###    antpairs <- 1; ### to define 
+###    if (is.null(additive.gamma.sum)) additive.gamma.sum <- matrix(1,dim.rv,ptheta)
+###
+###
+###  if (pair.structure==1 & dep.model==3) { ## {{{ 
+###### something with dimensions of rv.des 
+###### theta.des
+###       antpairs <- nrow(pairs); 
+###       if ( (length(dim(theta.des))!=3)  & (length(dim(random.design))==3) )
+###       {
+###          Ptheta.des <- array(0,c(nrow(theta.des),ncol(theta.des),antpairs))
+###          for (i in 1:antpairs) Ptheta.des[,,i] <- theta.des
+###       theta.des <- Ptheta.des
+###       }
+###       if ( (length(dim(theta.des))==3)  & (length(dim(random.design))!=3) )
+###       {
+###           rv.des <- array(0,c(2,ncol(random.design),antpairs))
+###           for (i in 1:antpairs) {
+###		   rv.des[1,,i] <- random.design[pairs[i,1],]
+###		   rv.des[2,,i] <- random.design[pairs[i,2],]
+###	   }
+###       random.design <- rv.des
+###       }
+###       if ( (length(dim(theta.des))!=3)  & (length(dim(random.design))!=3) )
+###       {
+###          Ptheta.des <- array(0,c(nrow(theta.des),ncol(theta.des),antpairs))
+###          rv.des <- array(0,c(2,ncol(random.design),antpairs))
+###          for (i in 1:antpairs) {
+###		   rv.des[1,,i] <- random.design[pairs[i,1],]
+###		   rv.des[2,,i] <- random.design[pairs[i,2],]
+###                   Ptheta.des[,,i] <- theta.des
+###	   }
+###       theta.des <- Ptheta.des
+###       random.design <- rv.des
+###       }
+###
+###       if (max(pairs)>antpers) stop("Indices of pairs should refer to given data \n"); 
+###       if (is.null(pairs.rvs)) pairs.rvs <- rep(dim(random.design)[2],antpairs)
+######       if (max(pairs.rvs)> dim(random.design)[3] | max(pairs.rvs)>ncol(theta.des[1,,])) 
+######	       stop("random variables for each cluster higher than  possible, pair.rvs not consistent with random.design or theta.des\n"); 
+###       clusterindex <- pairs-1; 
+###  } ## }}} 
+###
+###    if (pair.structure==1 & dep.model!=3) {
+###        clusterindex <- pairs-1; 
+###        antpairs <- nrow(pairs); 
+###        pairs.rvs <- 1
+###     }
+###
+###    if (pair.structure==1) {
+###	    if (length(case.control)!=antpairs)         case.control <- rep(case.control[1],antpairs)
+###	    if (length(pair.ascertained)!=antpairs) pair.ascertained <- rep(pair.ascertained[1],antpairs)
+###	    if (any(case.control+pair.ascertained==2))  stop("Each pair is either case.control pair or pair.ascertained \n"); 
+###    }
+###
+###    if (is.null(Dbeta.iid)) Dbeta.iid <- matrix(0,length(cause),1); 
+###    ptrunc <- rep(1,antpers); 
+#### }}}
+###
 
-    if (is.null(theta.des)==TRUE) ptheta<-1; 
-    if (is.null(theta.des)==TRUE) theta.des<-matrix(1,antpers,ptheta) ###    else theta.des<-as.matrix(theta.des); 
-###    ptheta<-ncol(theta.des); 
-###    if (nrow(theta.des)!=antpers) stop("Theta design does not have correct dim");
+  ### setting design for random variables, in particular with pairs are given
+  ddd <- randomDes(dep.model,random.design,theta.des,theta,antpers,additive.gamma.sum,pairs,pairs.rvs,var.link,clusterindex)
 
-  if (!is.null(pairs)) { pair.structure <- 1; } else  pair.structure <- 0;  
-  if (length(dim(theta.des))==3) ptheta<-dim(theta.des)[2] else if (length(dim(theta.des))==2) ptheta<-ncol(theta.des)
-  if  (nrow(theta.des)!=antpers & dep.model!=3 & pair.structure==0 ) stop("Theta design does not have correct dim");
-  if  (nrow(theta.des)!=nn      & dep.model!=3 & pair.structure==1 ) stop("Theta design does not have correct dim");
+###  ddd <- randomDes2DIM(dep.model,random.design,theta.des,theta,antpers,additive.gamma.sum,pairs,pairs.rvs,var.link,clusterindex)
+###  print(ddd)
 
-   if (length(dim(theta.des))!=3) theta.des <- as.matrix(theta.des)
-###   theta.des <- as.matrix(theta.des)
+  random.design=ddd$random.design;clusterindex=ddd$clusterindex;
+  antpairs=ddd$antpairs; pairs.rvs=ddd$pairs.rvs;
+  theta=ddd$theta;ptheta=ddd$ptheta;theta.des=ddd$theta.des
+  pair.structure=ddd$pair.structure; dep.model=ddd$dep.model
+  dim.rv <- ddd$dim.rv; additive.gamma.sum=ddd$additive.gamma.sum
 
-    dimbeta <- length(beta); 
-    if (is.null(theta)==TRUE) {
-        if (var.link==1) theta<- rep(-0.7,ptheta);  
-        if (var.link==0) theta<- rep(exp(-0.7),ptheta);   
-    }       
-    if (length(theta)!=ptheta) theta<-rep(theta[1],ptheta); 
-    theta.score<-rep(0,ptheta);Stheta<-var.theta<-matrix(0,ptheta,ptheta); 
+###  print(dep.model); print(theta); print(head(theta.des)); 
 
-    if (maxclust==1) stop("No clusters, maxclust size=1\n"); 
+  if (dep.model==3) model <- "clayton.oakes"
 
-    antpairs <- 1; ### to define 
-    if (is.null(additive.gamma.sum)) additive.gamma.sum <- matrix(1,dim.rv,ptheta)
-
-
-  if (pair.structure==1 & dep.model==3) { ## {{{ 
-### something with dimensions of rv.des 
-### theta.des
-       antpairs <- nrow(pairs); 
-       if ( (length(dim(theta.des))!=3)  & (length(dim(random.design))==3) )
-       {
-          Ptheta.des <- array(0,c(nrow(theta.des),ncol(theta.des),antpairs))
-          for (i in 1:antpairs) Ptheta.des[,,i] <- theta.des
-       theta.des <- Ptheta.des
-       }
-       if ( (length(dim(theta.des))==3)  & (length(dim(random.design))!=3) )
-       {
-           rv.des <- array(0,c(2,ncol(random.design),antpairs))
-           for (i in 1:antpairs) {
-		   rv.des[1,,i] <- random.design[pairs[i,1],]
-		   rv.des[2,,i] <- random.design[pairs[i,2],]
-	   }
-       random.design <- rv.des
-       }
-       if ( (length(dim(theta.des))!=3)  & (length(dim(random.design))!=3) )
-       {
-          Ptheta.des <- array(0,c(nrow(theta.des),ncol(theta.des),antpairs))
-          rv.des <- array(0,c(2,ncol(random.design),antpairs))
-          for (i in 1:antpairs) {
-		   rv.des[1,,i] <- random.design[pairs[i,1],]
-		   rv.des[2,,i] <- random.design[pairs[i,2],]
-                   Ptheta.des[,,i] <- theta.des
-	   }
-       theta.des <- Ptheta.des
-       random.design <- rv.des
-       }
-
-       if (max(pairs)>antpers) stop("Indices of pairs should refer to given data \n"); 
-       if (is.null(pairs.rvs)) pairs.rvs <- rep(dim(random.design)[2],antpairs)
-###       if (max(pairs.rvs)> dim(random.design)[3] | max(pairs.rvs)>ncol(theta.des[1,,])) 
-###	       stop("random variables for each cluster higher than  possible, pair.rvs not consistent with random.design or theta.des\n"); 
-       clusterindex <- pairs-1; 
-  } ## }}} 
-
-    if (pair.structure==1 & dep.model!=3) {
-        clusterindex <- pairs-1; 
-        antpairs <- nrow(pairs); 
-        pairs.rvs <- 1
-     }
-
-    if (pair.structure==1) {
-	    if (length(case.control)!=antpairs)         case.control <- rep(case.control[1],antpairs)
-	    if (length(pair.ascertained)!=antpairs) pair.ascertained <- rep(pair.ascertained[1],antpairs)
-	    if (any(case.control+pair.ascertained==2))  stop("Each pair is either case.control pair or pair.ascertained \n"); 
+if (pair.structure==1) {
+    if (length(case.control)!=antpairs)         case.control <- rep(case.control[1],antpairs)
+    if (length(pair.ascertained)!=antpairs) pair.ascertained <- rep(pair.ascertained[1],antpairs)
+    if (any(case.control+pair.ascertained==2))  stop("Each pair is either case.control pair or pair.ascertained \n"); 
     }
 
     if (is.null(Dbeta.iid)) Dbeta.iid <- matrix(0,length(cause),1); 
     ptrunc <- rep(1,antpers); 
-# }}}
 
-    ## }}}
-
-    if (dep.model==3) model <- "clayton.oakes"
-
-###  print(dep.model)
-###  print(summary(case.control))
-###  print(length(case.control))
-###  print(summary(pair.ascertained))
-###  print(length(pair.ascertained))
-###  print(dim(random.design)); print(dim(theta.des)); 
-###  print(dim(additive.gamma.sum))
-###  print(head(pairs.rvs))
-###  print(antpairs)
+    theta.score<-rep(0,ptheta);Stheta<-var.theta<-matrix(0,ptheta,ptheta); 
 
     loglike <- function(par) 
     { ## {{{
@@ -384,7 +399,7 @@ binomial.twostage <- function(margbin,data=sys.parent(),
       if (pair.structure==0 | dep.model!=3) Xtheta <- as.matrix(theta.des) %*% matrix(c(par[seq(1,ptheta)]),nrow=ptheta,ncol=1);
       if (pair.structure==1 & dep.model==3) Xtheta <- matrix(0,antpers,1); ## not needed 
       if (pair.structure==1 & dep.model!=3) Xtheta <- as.matrix(theta.des) %*% matrix(c(par[seq(1,ptheta)]),nrow=ptheta,ncol=1);
-      DXtheta <- array(0,c(1,1,1));
+      DXtheta <- matrix(0,1,1);
 
       if (twostage==0) epar <- par[seq(1,ptheta)] else epar <- par
       if (twostage==0) { ### update, marginal.p og score for logistic model
@@ -670,10 +685,10 @@ binomial.twostage <- function(margbin,data=sys.parent(),
     if (dep.model==3 & pair.structure==1) attr(ud, "likepairs") <- c(out$likepairs)
     if (dep.model==3 & pair.structure==0) attr(ud, "pardes") <- theta.des
     if (dep.model==3 & pair.structure==0) attr(ud, "theta.des") <- theta.des
-    if (dep.model==3 & pair.structure==1) attr(ud, "pardes") <-    theta.des[,,1,drop=FALSE]
-    if (dep.model==3 & pair.structure==1) attr(ud, "theta.des") <- theta.des[,,1,drop=FALSE]
+    if (dep.model==3 & pair.structure==1) attr(ud, "pardes") <-    theta.des[,,1]
+    if (dep.model==3 & pair.structure==1) attr(ud, "theta.des") <- theta.des[,,1]
     if (dep.model==3 & pair.structure==0) attr(ud, "rv1") <-    random.design[1,,drop=FALSE]
-    if (dep.model==3 & pair.structure==1) attr(ud, "rv1") <-    random.design[1,,1,drop=FALSE]
+    if (dep.model==3 & pair.structure==1) attr(ud, "rv1") <-    random.design[,,1]
  
     attr(ud, "response") <- "binomial"
     return(ud);
@@ -690,7 +705,7 @@ p11.binomial.twostage.RV <- function(theta,rv1,rv2,p1,p2,pardes,ags=NULL,link=0,
 } ## }}} 
 
 ##' @export
-concordance.twostage<- function(theta,p,rv1,rv2,theta.des,additive.gamma.sum=NULL,link=0,var.par=0)
+concordanceTwostage<- function(theta,p,rv1,rv2,theta.des,additive.gamma.sum=NULL,link=0,var.par=0,...)
 {# {{{
 
    ### takes dependence paramter from output
@@ -726,7 +741,7 @@ concordance.twostage<- function(theta,p,rv1,rv2,theta.des,additive.gamma.sum=NUL
 }	# }}}
 
 ##' @export
-concordance.twin.ace<- function(object,rv1=NULL,rv2=NULL,xmarg=NULL,type="ace")
+concordanceTwinACE<- function(object,rv1=NULL,rv2=NULL,xmarg=NULL,type="ace",...)
 {# {{{
 
 if (type=="ace" | type=="ade") {
@@ -1185,6 +1200,9 @@ simbinClaytonOakes.family.ace <- function(K,varg,varc,beta=NULL,alpha=NULL)  ##
   ## K antal clustre (families), n=antal i clustre
 ###  K <- 10000
   n <- 4 # family members with ace structure
+  sumpar <- sum(varg+varc)
+  varg <- varg/sumpar^2; 
+  varc <- varc/sumpar^2
   ## total variance 1/(varg+varc)
   ## {{{  random effects 
   ###  means varg/(varg+varc) and variances varg/(varg+varc)^2
@@ -1200,7 +1218,7 @@ simbinClaytonOakes.family.ace <- function(K,varg,varc,beta=NULL,alpha=NULL)  ##
   child2 <- apply(cbind(mother.g[,c(1,3)],father.g[,c(1,3)]),1,sum) + env
   Gam1 <- cbind(mother,father,child1,child2)
   ## }}} 
-  apply(Gam1,2,mean); apply(Gam1,2,var); cor(Gam1)
+###  apply(Gam1,2,mean); apply(Gam1,2,var); cor(Gam1)
 
   ## {{{ marginals p's and conditional p's given random effects 
   ### marginals p's for mother, father, children
@@ -1241,6 +1259,9 @@ simbinClaytonOakes.twin.ace <- function(K,varg,varc,beta=NULL,alpha=NULL)  ## {{
 ###  K <- 10000
   n <- 2 # twins with ace structure
   ## total variance 1/(varg+varc)
+  sumpar <- sum(varg+varc)
+  varg <- varg/sumpar^2; 
+  varc <- varc/sumpar^2
   ## {{{  random effects 
   ###  means varg/(varg+varc) and variances varg/(varg+varc)^2
   eta <- varc+varg
@@ -1254,6 +1275,7 @@ simbinClaytonOakes.twin.ace <- function(K,varg,varc,beta=NULL,alpha=NULL)  ## {{
   dz1 <- apply(dz.g[,c(1,2)],1,sum) + env2
   dz2 <- apply(dz.g[,c(1,3)],1,sum) + env2
   Gam1 <- rbind(cbind(mz,mz),cbind(dz1,dz2))
+###  print(apply(Gam1,2,mean)); print(apply(Gam1,2,var))
   ## }}} 
 
   ## {{{ marginals p's and conditional p's given random effects 
