@@ -105,7 +105,7 @@ RcppExport SEXP FastCoxPrepStrata(SEXP EntrySEXP,
 		SEXP offsetsSEXP, 
 		SEXP ZSEXP,
 		SEXP caseweightsSEXP
-		) {
+		) {/*{{{*/
 	BEGIN_RCPP
 		arma::vec Entry = Rcpp::as<arma::vec>(EntrySEXP);
 	arma::vec  Exit  = Rcpp::as<arma::vec>(ExitSEXP);
@@ -174,6 +174,7 @@ RcppExport SEXP FastCoxPrepStrata(SEXP EntrySEXP,
 		// vec Entry = Rcpp::as<vec>(entry);  
 		Exit.insert_rows(0,Entry);
 		X.insert_rows(0,X);
+		Z.insert_rows(0,Z);
 		Status.insert_rows(0,Status);
 		Id.insert_rows(0,Id);
 		strata.insert_rows(0,strata);
@@ -211,6 +212,9 @@ RcppExport SEXP FastCoxPrepStrata(SEXP EntrySEXP,
 		XX = XX.rows(idx);
 		X = X.rows(idx);  
 	}
+	if (Z.n_rows==X.n_rows) {
+		Z = Z.rows(idx);  
+	}
 	if ((ZX.n_rows==XX.n_rows) & (XX.n_rows>0)) {
 		ZX = ZX.rows(idx);  
 	}
@@ -243,10 +247,11 @@ RcppExport SEXP FastCoxPrepStrata(SEXP EntrySEXP,
 					Rcpp::Named("caseweights")=caseweights,
 					Rcpp::Named("offset")=offsets,				       
 					Rcpp::Named("strata")=strata,				       
-					Rcpp::Named("ZX")=ZX				       
+					Rcpp::Named("ZX")=ZX,				       
+					Rcpp::Named("Z")=Z				       
 					)));
 	END_RCPP
-}
+}/*}}}*/
 
 colvec  whichi(IntegerVector a,int n, int j) {
 	colvec res(n); 
@@ -408,7 +413,6 @@ colvec  cumsumstrataPO(colvec w,colvec S0,IntegerVector strata,int nstrata,doubl
 } 
 
 RcppExport SEXP cumsumstrataPOR(SEXP iw,SEXP iS0,SEXP istrata,SEXP instrata,SEXP ipropodds,SEXP iexb) {
-
 	colvec w = Rcpp::as<colvec>(iw);
 	colvec S0 = Rcpp::as<colvec>(iS0);
 	colvec exb = Rcpp::as<colvec>(iexb);
@@ -523,6 +527,201 @@ RcppExport SEXP revcumsumstrataR(SEXP ia,SEXP istrata, SEXP instrata) {
 	return(rres);
 } 
 
+/*{{{   Double-Fine-Gray recursive utility functions  */
+
+RcppExport SEXP cumsumstrataDFGR(SEXP iw,SEXP iS0,SEXP icause,SEXP istrata,SEXP instrata,SEXP iexb) {
+	colvec w = Rcpp::as<colvec>(iw);
+	mat S0 = Rcpp::as<mat>(iS0);
+	mat exb = Rcpp::as<mat>(iexb);
+	IntegerVector strata(istrata); 
+	IntegerVector cause(icause); 
+	int nstrata = Rcpp::as<int>(instrata);
+
+        unsigned n = S0.n_rows;
+	colvec tmpsum1(nstrata); tmpsum1.zeros(); 
+	colvec tmpsum2(nstrata); tmpsum2.zeros(); 
+	mat res = S0; 
+	colvec pow1(n); 
+	colvec pow2(n); 
+
+	for (unsigned i=0; i<n; i++) {
+		int ss=strata(i); 
+		if ((ss<nstrata) & (ss>=0))  {
+			double F1=(1-exp(-exb(i,0)*tmpsum1(ss))); // F_1
+			double F2=(1-exp(-exb(i,1)*tmpsum2(ss))); // F_2 
+			pow1(i)=(1-F1-F2)/(1-F1); 
+			pow2(i)=(1-F1-F2)/(1-F2); 
+			if (cause(i)==1) tmpsum1(ss) += pow1(i)*w(i)/S0(i,0); 
+			if (cause(i)==2) tmpsum2(ss) += pow2(i)*w(i)/S0(i,1); 
+			res(i,0) = tmpsum1(ss);
+			res(i,1) = tmpsum2(ss);
+		}
+	}  
+
+	List rres; 
+	rres["base"]=res; 
+	rres["pow1"]=pow1; 
+	rres["pow2"]=pow2; 
+	return(rres);
+}
+	
+RcppExport SEXP cumsumstrataDFGRestrictR(SEXP iw,SEXP iS0,SEXP icause,SEXP istrata,SEXP instrata,SEXP iexb,SEXP iLam1inf) {
+	colvec w = Rcpp::as<colvec>(iw);
+	mat S0 = Rcpp::as<mat>(iS0);
+	mat exb = Rcpp::as<mat>(iexb);
+	IntegerVector strata(istrata); 
+	IntegerVector cause(icause); 
+	int nstrata = Rcpp::as<int>(instrata);
+	colvec Lam1inf = Rcpp::as<colvec>(iLam1inf);
+
+        unsigned n = S0.n_rows;
+	colvec tmpsum1(nstrata); tmpsum1.zeros(); 
+	colvec tmpsum2(nstrata); tmpsum2.zeros(); 
+	mat res = S0; 
+	colvec pow1(n); 
+	colvec pow2(n); 
+
+	for (unsigned i=0; i<n; i++) {
+		int ss=strata(i); 
+		if ((ss<nstrata) & (ss>=0))  {
+			double F1=(1-exp(-exb(i,0)*tmpsum1(ss)));                            // F_1
+			double F2=(1-exp(-exb(i,1)*tmpsum2(ss)))*exp(-exb(i,0)*Lam1inf(ss)); // F_2 
+			pow1(i)=(1-F1-F2)/(1-F1); 
+			pow2(i)=(1-F1-F2)/(exp(-exb(i,1)*tmpsum2(ss))*exp(-exb(i,0)*Lam1inf(ss)));                       ; 
+			if (cause(i)==1) tmpsum1(ss) += pow1(i)*w(i)/S0(i,0); 
+			if (cause(i)==2) tmpsum2(ss) += pow2(i)*w(i)/S0(i,1); 
+			res(i,0) = tmpsum1(ss);
+			res(i,1) = tmpsum2(ss);
+		}
+	}  
+
+	for (int i=0;i<nstrata; i++) Lam1inf(i)=tmpsum1(i); 
+
+	List rres; 
+	rres["base"]=res; 
+	rres["pow1"]=pow1; 
+	rres["pow2"]=pow2; 
+	rres["Lam1inf"]=Lam1inf; 
+	return(rres);
+}
+	
+RcppExport SEXP DLambetaDFGR(SEXP iweights,SEXP iS0,SEXP icause,SEXP iE,SEXP iXi,SEXP istrata,SEXP instrata,SEXP iexb) {
+//	colvec w = Rcpp::as<colvec>(iw);
+	colvec weights = Rcpp::as<colvec>(iweights);
+	mat S0 = Rcpp::as<mat>(iS0);
+	mat exb = Rcpp::as<mat>(iexb);
+	mat E = Rcpp::as<mat>(iE);
+	mat Xi = Rcpp::as<mat>(iXi);
+	IntegerVector strata(istrata); 
+	IntegerVector cause(icause); 
+	int nstrata = Rcpp::as<int>(instrata);
+
+        unsigned n = S0.n_rows;
+	unsigned p = E.n_cols/2;
+
+	colvec tmpsum1(nstrata); tmpsum1.zeros(); 
+	colvec tmpsum2(nstrata); tmpsum2.zeros(); 
+	colvec pow1(n); colvec pow2(n); 
+	mat dLam1(n,p); dLam1.zeros(); 
+	mat dLam12(n,p); dLam12.zeros(); 
+	mat dLam2(n,p); dLam2.zeros(); 
+	mat dLam21(n,p); dLam21.zeros(); 
+	mat dLam1tminus(nstrata,p); dLam1tminus.zeros(); 
+	mat dLam12tminus(nstrata,p); dLam12tminus.zeros(); 
+	mat dLam2tminus(nstrata,p); dLam2tminus.zeros(); 
+	mat dLam21tminus(nstrata,p); dLam21tminus.zeros(); 
+
+       for (unsigned i=0; i<n; i++) {
+		int ss=strata(i); 
+		double F1=(1-exp(-exb(i,0)*tmpsum1(ss))); // F_1
+		double F2=(1-exp(-exb(i,1)*tmpsum2(ss))); // F_2 
+		pow1(i)=(1-F1-F2)/(1-F1); 
+		pow2(i)=(1-F1-F2)/(1-F2); 
+
+        	if (cause(i)==1)  {
+		        dLam1.row(i)=dLam1tminus.row(ss)+weights(i)*
+			( -(dLam1tminus.row(ss)*exb(i)+Xi.row(i)*exb(i,0))/S0(i)-E.row(i)*pow1(i)/S0(i));
+		        dLam12.row(i)=dLam12tminus.row(ss)+weights(i)*
+			( -((1-F2)/(1-F1))*(dLam12tminus.row(ss)*exb(i)+Xi.row(i)*exb(i,0))/S0(i)-E.row(i)*pow1(i)/S0(i));
+			tmpsum1(ss) += pow1(i)*weights(i)/S0(i,0); 
+		        dLam1tminus.row(ss) = dLam1.row(i); 
+		        dLam12tminus.row(ss) = dLam12.row(i); 
+		}
+
+		if (cause(i)==2) {
+		        dLam2.row(i) = dLam2tminus.row(ss)+weights(i)*
+			( -(dLam2tminus.row(ss)*exb(i)+Xi.row(i)*(pow2(i)-1))/S0(i)-E.row(i)*pow2(i)/S0(i));
+                        dLam21.row(i) = dLam21tminus.row(ss)+weights(i)*
+			( -((1-F1)/(1-F2))*(dLam21tminus.row(ss)*exb(i)+Xi.row(i)*(pow2(i)-1))/S0(i)-E.row(i)*pow2(i)/S0(i));
+			tmpsum2(ss) += pow2(i)*weights(i)/S0(i,1); 
+		        dLam2tminus.row(ss) = dLam2.row(i); 
+		        dLam21tminus.row(ss) = dLam21.row(i); 
+		}
+	}  
+
+	List rres; 
+	rres["DLam1"]=dLam1; 
+	rres["DLam12"]=dLam12; 
+	rres["DLam2"]=dLam2; 
+	rres["DLam21"]=dLam21; 
+	return(rres);
+}
+/*}}}*/
+
+RcppExport SEXP wherestrataR(SEXP ir,SEXP ia,SEXP istrata, SEXP instrata) {
+	colvec a = Rcpp::as<colvec>(ia);
+	colvec r = Rcpp::as<colvec>(ir);
+	IntegerVector intstrata(istrata); 
+	int nstrata = Rcpp::as<int>(instrata);
+	unsigned n = a.n_rows;
+
+	colvec tmpsum(nstrata); tmpsum.zeros(); 
+	colvec nsum(nstrata); nsum.zeros(); 
+	colvec maxv(nstrata); maxv.zeros(); 
+	colvec minv(nstrata); minv.zeros(); 
+	for (unsigned i=0; i<n; i++) {
+		int ss=intstrata(i); 
+		double irs = r(ss);  
+		if ((a(i) > maxv(ss)) | (nsum(ss)==0)) maxv(ss)=a(i); 
+		if ((a(i) < minv(ss)) | (nsum(ss)==0)) minv(ss)=a(i); 
+		if (irs>a(i)) tmpsum(ss)=nsum(ss); 
+		nsum(ss)+=1; 
+	}
+
+	List rres; 
+	rres["where"]=tmpsum; 
+	rres["max"]=maxv; 
+	rres["min"]=minv; 
+	rres["nstrata"]=nsum; 
+	return(rres);
+}
+
+
+RcppExport SEXP maxminidR(SEXP ia,SEXP istrata, SEXP instrata) {
+	colvec a = Rcpp::as<colvec>(ia);
+	IntegerVector intstrata(istrata); 
+	int nstrata = Rcpp::as<int>(instrata);
+	unsigned n = a.n_rows;
+
+	colvec nsum(nstrata); nsum.zeros(); 
+	colvec maxv(nstrata); maxv.zeros(); 
+	colvec minv(nstrata); minv.zeros(); 
+	for (unsigned i=0; i<n; i++) {
+		int ss=intstrata(i); 
+		if ((a(i) > maxv(ss)) | (nsum(ss)==0)) maxv(ss)=a(i); 
+		if ((a(i) < minv(ss)) | (nsum(ss)==0)) minv(ss)=a(i); 
+		nsum(ss)+=1; 
+	}
+
+	List rres; 
+	rres["max"]=maxv; 
+	rres["min"]=minv; 
+	rres["nstrata"]=nsum; 
+	return(rres);
+}
+
+
+
 RcppExport SEXP riskstrataR(SEXP ia,SEXP istrata, SEXP instrata) {
 	colvec a = Rcpp::as<colvec>(ia);
 	IntegerVector intstrata(istrata); 
@@ -791,31 +990,31 @@ RcppExport SEXP cumsumidstratasumR(SEXP ia,SEXP iid,SEXP inid,SEXP istrata, SEXP
 		ressum(i) = cumsum(ss);
 		ressumid(i) = tmpsum(ss,lid);
 		tmpsqr(ss)=ressqu(i); 
-				}  
+	}  
 
-				List rres; 
-				rres["sumsquare"]=ressqu; 
-				rres["sum"]=ressum; 
-				rres["lagsum"]=lagressum; 
-				rres["sumidstrata"]=ressumid; 
-				rres["lagsumidstrata"]=lagressumid; 
-				return(rres);
-				} 
+List rres; 
+rres["sumsquare"]=ressqu; 
+rres["sum"]=ressum; 
+rres["lagsum"]=lagressum; 
+rres["sumidstrata"]=ressumid; 
+rres["lagsumidstrata"]=lagressumid; 
+return(rres);
+} 
 
 
-				colvec cumsumAS(const colvec &a,IntegerVector strata,int nstrata) {
-				unsigned n = a.n_rows;
-				colvec tmpsum(nstrata); tmpsum.zeros(); 
-				colvec ressum = a; 
-				double sums=0; 
-				for (unsigned i=0; i<n; i++) {
-				int ss=strata(i); 
-				ressum(i) = sums+a(i)-tmpsum(ss); 
-				tmpsum(ss)=a(i); 
-				sums=ressum(i); 
-				}  
-				return(ressum);
-				} 
+colvec cumsumAS(const colvec &a,IntegerVector strata,int nstrata) {
+unsigned n = a.n_rows;
+colvec tmpsum(nstrata); tmpsum.zeros(); 
+colvec ressum = a; 
+double sums=0; 
+for (unsigned i=0; i<n; i++) {
+int ss=strata(i); 
+ressum(i) = sums+a(i)-tmpsum(ss); 
+tmpsum(ss)=a(i); 
+sums=ressum(i); 
+}  
+return(ressum);
+} 
 
 RcppExport SEXP cumsumASR(SEXP ia, SEXP istrata, SEXP instrata) {
 	colvec a = Rcpp::as<colvec>(ia);
@@ -831,15 +1030,15 @@ RcppExport SEXP cumsumASR(SEXP ia, SEXP istrata, SEXP instrata) {
 	double sums=0; 
 	for (unsigned i=0; i<n; i++) {
 		int ss=intstrata(i); 
-		Rprintf(" %ld %ld %ld \n",n,ss,i); 
+//		Rprintf(" %ld %ld %ld \n",n,ss,i); 
 		ressum(i) = sums+a(i)-tmpsum(ss); 
 		tmpsum(ss)=a(i); 
 		sums=ressum(i); 
 	}  
 
-	List rres; 
-	rres["sum"]=ressum; 
-	return(rres);
+List rres; 
+rres["sum"]=ressum; 
+return(rres);
 } 
 
 
@@ -1114,7 +1313,7 @@ RcppExport SEXP FastCoxPLstrata(SEXP betaSEXP,
 		SEXP ZXSEXP,
 		SEXP caseweightsSEXP) {
 	BEGIN_RCPP
-		colvec beta = Rcpp::as<colvec>(betaSEXP);
+	colvec beta = Rcpp::as<colvec>(betaSEXP);
 	mat X = Rcpp::as<mat>(XSEXP);
 	mat XX = Rcpp::as<mat>(XXSEXP);
 	mat ZX = Rcpp::as<mat>(ZXSEXP);
@@ -1206,7 +1405,7 @@ RcppExport SEXP FastCoxPLstrataPO(SEXP betaSEXP,
 		SEXP offsetsSEXP,
 		SEXP ZXSEXP,
 		SEXP propoddsSEXP) {
-	BEGIN_RCPP
+	BEGIN_RCPP/*{{{*/
 		colvec beta = Rcpp::as<colvec>(betaSEXP);
 	mat X = Rcpp::as<mat>(XSEXP);
 	mat XX = Rcpp::as<mat>(XXSEXP);
@@ -1299,7 +1498,7 @@ RcppExport SEXP FastCoxPLstrataPO(SEXP betaSEXP,
 				Rcpp::Named("DLam")=DLam 
 				));
 	END_RCPP
-}
+} /*}}}*/
 
 
 RcppExport SEXP FastCoxPLstrataAddGam(SEXP betaSEXP,
@@ -1445,15 +1644,28 @@ RcppExport SEXP FastCoxPLstrataAddGam(SEXP betaSEXP,
 }
 
 
-RcppExport SEXP Matdoubleindex(SEXP im,SEXP irow,SEXP icols,SEXP ilength) {
+RcppExport SEXP Matdoubleindex(SEXP im,SEXP irow,SEXP icols,SEXP ilength,SEXP iassign,SEXP ixvec ) {
 	mat m = Rcpp::as<mat>(im);
 	IntegerVector rows(irow); 
 	IntegerVector cols(icols); 
+	colvec xvec = Rcpp::as<colvec>(ixvec);
 	unsigned l = Rcpp::as<int>(ilength);
+	unsigned assign = Rcpp::as<int>(iassign);
 	vec res(l); 
-	for (unsigned i=0; i<l; i++) res(i)=m(rows(i),cols(i)); 
+	vec where(l); 
 	List rres; 
+
+	int mn = m.n_rows;
+	int mp = m.n_cols;
+
+	for (unsigned i=0; i<l; i++) where(i)= ((rows(i) > -1) & (rows(i)< mn)) * ((cols(i) > -1) & (cols(i)< mp)) ;
+	if (assign==0)  {
+	for (unsigned i=0; i<l; i++) if (where(i)>0) { res(i)=m(rows(i),cols(i)); } else res(i)=0; 
 	rres["mat"]=res; 
+	} else  {
+	for (unsigned i=0; i<l; i++) if (where(i)>0) { m(rows(i),cols(i))=xvec(i);  }
+	rres["mat"]=m; 
+	}
 	return(rres);
 } 
 
@@ -1471,17 +1683,21 @@ mat CubeVecC(mat XX, vec beta,int dim1) {
 RcppExport SEXP CubeVec(SEXP XXSEXP, SEXP betaSEXP)
 {
 	BEGIN_RCPP
-		colvec beta = Rcpp::as<colvec>(betaSEXP);
+//		colvec beta = Rcpp::as<colvec>(betaSEXP);
 	mat XX = Rcpp::as<mat>(XXSEXP);
-	unsigned p = beta.n_rows;
+	mat beta = Rcpp::as<mat>(betaSEXP);
+	unsigned p = beta.n_cols;
 	unsigned n = XX.n_rows;
 
 	mat XXbeta(n,p);
+	mat iXXbeta(n,p*p);
 	for (unsigned j=0; j<n; j++)  {
-		XXbeta.row(j)=(reshape(XX.row(j),p,p)*beta).t();
+		mat iXX= pinv(reshape(XX.row(j),p,p)); 
+		iXXbeta.row(j)=vectorise(iXX).t();
+		XXbeta.row(j)=(iXX*(beta.row(j)).t()).t();
 	}
 
-	return(Rcpp::List::create(Rcpp::Named("XXbeta")=XXbeta));
+	return(Rcpp::List::create(Rcpp::Named("XXbeta")=XXbeta,Rcpp::Named("iXX")=iXXbeta));
 	END_RCPP
 }
 
@@ -1592,7 +1808,7 @@ RcppExport SEXP PropTestCoxClust(SEXP iU, SEXP idUt, SEXP iwrr, SEXP iZ,
 		SEXP inclust,SEXP iid, SEXP istrata,SEXP instrata,
 		SEXP iJumps) {
 	BEGIN_RCPP
-		mat U      = Rcpp::as<mat>(iU);
+	mat U      = Rcpp::as<mat>(iU);
 	mat dUt = Rcpp::as<mat>(idUt);
 	arma::vec wrr = Rcpp::as<arma::vec>(iwrr);
 	arma::vec Lam = Rcpp::as<arma::vec>(iLam);
