@@ -52,7 +52,7 @@ extern SEXP CubeVec(SEXP, SEXP);
 extern SEXP vecMatMat(SEXP, SEXP);
 extern SEXP OutCov(SEXP, SEXP);
 extern SEXP MatxCube(SEXP, SEXP, SEXP);
-extern SEXP Matdoubleindex(SEXP, SEXP, SEXP,SEXP);
+extern SEXP Matdoubleindex(SEXP, SEXP, SEXP,SEXP,SEXP,SEXP);
 extern SEXP CubeMat(SEXP, SEXP);
 extern SEXP PropTestCox(SEXP, SEXP,SEXP,SEXP);
 extern SEXP PropTestCoxClust(SEXP, SEXP, SEXP, SEXP,SEXP,SEXP, SEXP, SEXP, SEXP, SEXP,SEXP,SEXP, SEXP);
@@ -66,9 +66,14 @@ extern SEXP revcumsumidstratasumR(SEXP,SEXP, SEXP,SEXP, SEXP);
 extern SEXP revcumsumidstratasumCovR(SEXP,SEXP,SEXP, SEXP,SEXP, SEXP);
 extern SEXP cumsumstrataR(SEXP,SEXP, SEXP);
 extern SEXP cumsumstrataPOR(SEXP,SEXP,SEXP,SEXP,SEXP,SEXP);
+extern SEXP cumsumstrataDFGR(SEXP,SEXP,SEXP,SEXP,SEXP,SEXP);
+extern SEXP cumsumstrataDFGRestrictR(SEXP,SEXP,SEXP,SEXP,SEXP,SEXP,SEXP);
 extern SEXP DLambetaR(SEXP,SEXP,SEXP,SEXP,SEXP,SEXP,SEXP,SEXP);
+extern SEXP DLambetaDFGR(SEXP,SEXP,SEXP,SEXP,SEXP,SEXP,SEXP,SEXP);
 extern SEXP riskstrataR(SEXP,SEXP,SEXP); 
 extern SEXP meanriskR(SEXP,SEXP,SEXP,SEXP,SEXP);
+extern SEXP wherestrataR(SEXP,SEXP,SEXP,SEXP);
+extern SEXP maxminidR(SEXP,SEXP,SEXP);
 extern SEXP cumsumstratasumR(SEXP,SEXP, SEXP);
 extern SEXP cumsumidstratasumR(SEXP,SEXP, SEXP,SEXP, SEXP);
 extern SEXP cumsumASR(SEXP,SEXP, SEXP);
@@ -78,6 +83,7 @@ extern SEXP covrfstrataR(             SEXP,SEXP,SEXP,SEXP,SEXP, SEXP);
 extern SEXP covrfstrataCovR(SEXP,SEXP,SEXP,SEXP,SEXP,SEXP,SEXP, SEXP);
 extern SEXP sumstrataR(SEXP,SEXP, SEXP);
 extern SEXP XBmindex(SEXP,SEXP, SEXP);
+extern SEXP RVpairs2DIMRV(SEXP, SEXP);
 //extern SEXP backfitEaEt(SEXP,SEXP, SEXP,SEXP,SEXP, SEXP, SEXP, SEXP,SEXP);
 
 static const R_CallMethodDef CallEntries[] = {
@@ -109,10 +115,12 @@ static const R_CallMethodDef CallEntries[] = {
     {"FastPattern",                       (DL_FUNC) &FastPattern,                        3},
     {"MatxCube",                          (DL_FUNC) &MatxCube,                           3},
     {"meanriskR",                        (DL_FUNC) &meanriskR,                           5},
-    {"Matdoubleindex",                    (DL_FUNC) &Matdoubleindex,                     4},
-    {"_mets_ApplyBy",                      (DL_FUNC) &_mets_ApplyBy,                     3},
-    {"_mets_ApplyBy2",                     (DL_FUNC) &_mets_ApplyBy2,                    8},
-    {"_mets_loglikMVN",                    (DL_FUNC) &_mets_loglikMVN,                   13},
+    {"wherestrataR",                     (DL_FUNC) &wherestrataR,                        4},
+    {"maxminidR",                        (DL_FUNC) &maxminidR,                           3},
+    {"Matdoubleindex",                   (DL_FUNC) &Matdoubleindex,                     6},
+    {"_mets_ApplyBy",                    (DL_FUNC) &_mets_ApplyBy,                     3},
+    {"_mets_ApplyBy2",                   (DL_FUNC) &_mets_ApplyBy2,                    8},
+    {"_mets_loglikMVN",                  (DL_FUNC) &_mets_loglikMVN,                   13},
     {"_mets_RcppExport_registerCCallable", (DL_FUNC) &_mets_RcppExport_registerCCallable,0},
     {"pBhat",                             (DL_FUNC) &pBhat,                              6},
     {"PropTestCox",                       (DL_FUNC) &PropTestCox,                        4},
@@ -125,7 +133,10 @@ static const R_CallMethodDef CallEntries[] = {
     {"revcumsumstratasumR",               (DL_FUNC) &revcumsumstratasumR,                3},
     {"cumsumstratasumR",                  (DL_FUNC) &cumsumstratasumR,                   3},
     {"cumsumstrataPOR",                   (DL_FUNC) &cumsumstrataPOR,                    6},
+    {"cumsumstrataDFGR",                  (DL_FUNC) &cumsumstrataDFGR,                   6},
+    {"cumsumstrataDFGRestrictR",          (DL_FUNC) &cumsumstrataDFGRestrictR,           7},
     {"DLambetaR",                         (DL_FUNC) &DLambetaR,                          8},
+    {"DLambetaDFGR",                      (DL_FUNC) &DLambetaDFGR,                       8},
     {"cumsumidstratasumR",                (DL_FUNC) &cumsumidstratasumR,                 5},
     {"cumsumASR",                         (DL_FUNC) &cumsumidstratasumR,                 3},
     {"tailstrataR",                       (DL_FUNC) &tailstrataR,                        3},
@@ -150,8 +161,9 @@ static const R_CallMethodDef CallEntries[] = {
     {"Uhat",                              (DL_FUNC) &Uhat,                               5},
     {"uniprobit",                         (DL_FUNC) &uniprobit,                          8},
     {"vecMatMat",                         (DL_FUNC) &vecMatMat,                          2},
-    {"OutCov",                            (DL_FUNC) &OutCov,                          2},
+    {"OutCov",                            (DL_FUNC) &OutCov,                             2},
     {"XBmindex",                          (DL_FUNC) &XBmindex,                           3},
+    {"RVpairs2DIMRV",                     (DL_FUNC) &RVpairs2DIMRV,                      2},
     {NULL, NULL, 0}
 };
 
